import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * The alien flies around and tries to kill the tug.
 * 
 * @author David Osborne
 * @version 12/11/08
 */
public class Alien extends Ship
{
    int actDelay = 10;
    
    public Alien(){
        thrustSpeed = 0.2;
    }
    public void act() 
    {
        if(actDelay <= 0){
            ai();
        }
        else{
            actDelay--;
        }
        gravitate();
        moveNicely();
        
        if (isDead){
            GravityWorld world = (GravityWorld) getWorld();
            explode();
            world.checkForWin();
        }
        else if (hasCollided(Matter.class)){
            isDead = true;
        }
        else if (hasCollided(Laser.class)){
            isDead = true;
        }
    }
    
    /**
     * This method is responsible for the aliens' AI.
     * They try and counteract the tug's gravity generators by flying
     * towards or away form the tug.
     */
    public void ai(){
        List tugs = getWorld().getObjects(Tug.class);
        if(tugs.isEmpty() != true){
            
            // Finds the tug
            Tug tug = (Tug) tugs.get(0);
            Vector bearing = getBearing(tug);
            
            int tugDirection = DirectionUtil.normalise(bearing.getDirection());
            
            // If the alien's facing within 10 degrees of the tug
            boolean facingTug = (tugDirection <= facing+10 && tugDirection >= facing-10);
            
            if (bearing.getLength() <= 100){
                thrust();
                turnToDirection(DirectionUtil.normalise(tugDirection + 180));
            }
            else if (bearing.getLength() > 100){
                thrust();
                turnToDirection(tugDirection);
                
            }
            
            if (facingTug){
                if (Greenfoot.getRandomNumber(50) < 1){
                shoot();
            }
            }
        }
    }
    
    /**
     * These methods cause the alien to turnin a direction by a certain amount
     * The use of these methods makes sure the alien turns at a smooth rate.
     */
    public void turnLeft(){
        turn(-3);
    }
    public void turnRight(){
        turn(3);
    }
    
    /**
     * This method causes the alien to fire a laser shot
     * in the direction it's facing.
     */
    public void shoot(){
        Vector firing = new Vector(facing, 20);
        Laser laser = new Laser(firing, this);
        Vector laserOffset = new Vector(facing, 24);
        
        getWorld().addObject(laser, (int)(getX()+laserOffset.getX()),
                            (int)(getY() + laserOffset.getY()));
        
        Greenfoot.playSound("laser.wav");
    }
    
    /**
     * This method causes the alien to turn in the shortest way towards a given direction.
     */
    public void turnToDirection(int direction){
        int theta = facing - direction;
                if (theta <= 180 && theta > 0){
                    turnLeft();
                }
                else{
                    turnRight();
                }
    }
    
    /**
     * Returns true if the object has collided with another object of class cls.
     */
    public boolean hasCollided(Class cls){
        Object object = getOneIntersectingObject(cls);
        if (object != null && object != this){
            if (object.getClass() == Laser.class){
                if (((Laser) object).getShooter() == this){
                    return false;
                }
            }
            return true;
        }
        else{
        return false;
    }
    }
}
