import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
import java.awt.GraphicsEnvironment;

/**
 * Write a description of class GravityWorld here.
 * 
 * @author David Osborne
 * @version 4/11/08
 */
public class GravityWorld extends World
{
    Menu currentMenu;
    boolean gameIsRunning = false;
    ScoreKeeper scorekeeper;
    
    /**
     * Constructor for objects of class GravityWorld.
     */
    public GravityWorld()
    {    
        super(640, 480, 1);
        displayMainMenu();
        Greenfoot.startSimulation();
        Greenfoot.setSimulationSpeed(45);
    }


    /**
     * Generates a level with a certain number of objects randomly placed around the map
     * and one tug in the centre.
     */
    public void populateLevel(int numberOfAliens, int numberOfRocks){
        
        for(int a=0; a < numberOfAliens; a++){
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new Alien(), x, y);
        }
        
        for(int r=0; r < numberOfRocks; r++){
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(new Rock(), x, y);
        }
        if(Button.isSurvivalModeOn()){
            scorekeeper = new ScoreKeeper();
            addObject(scorekeeper, 0, 0);
            addObject(new SurvivalTug(), getWidth()/2, getHeight()/2);
            
        }
        else{
            addObject(new Tug(), getWidth()/2, getHeight()/2);
        }
        gameIsRunning = true;
    }
    
    /**
     * Gets rid of the current menu.
     */
    public void clearMenu(){
        if(currentMenu != null){
            currentMenu.clear();
            removeObject(currentMenu);
            currentMenu = null;
        }
    }
    public void clearAll(){
        for(Object thing : getObjects(Actor.class)){
            removeObject((Actor)thing);
        }
    }
    
    /**
     * Displays a new MainMenu.
     */
    public void displayMainMenu(){
        currentMenu = new MainMenu();
        addObject(currentMenu, 0, 0);
    }
    
    public void displayEndMenu(boolean won){
        currentMenu = new EndMenu(Button.isSurvivalModeOn(), won);
        addObject(currentMenu, 0, 0);
    }
    
    
    public void endGame(boolean won){
        if (gameIsRunning){
            gameIsRunning = false;
            displayEndMenu(won);
        }
    }
    
    /**
     * Sees if all the aliens have died yet.
     */
    public void checkForWin(){
            List aliens = getObjects(Alien.class);
            if (aliens.isEmpty()){
                endGame(true);
        }
    }
    
    
    public ScoreKeeper getScorekeeper(){
        return scorekeeper;
    }

}
